/*
 * Copyright (C) 2001 Yasuhiro Ohara
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.  
 */

#include "ospf6d.h"

/* RFC2328 12.4.3. Summary-LSA */
int
ospf6_abr_inter_router_add (void *data)
{
#if 0
  struct ospf6_lsa *lsa = data;
  struct ospf6_area *area = lsa->scope;
  struct ospf6_inter_area_router_lsa *ier;
  struct prefix_ls prefix_ls;
  struct ospf6_route_request reqest;
  struct ospf6_route *abr_entry;

  ier = (struct ospf6_inter_area_router_lsa *) (lsa->header + 1);

  prefix_ls.id = htonl (0);
  prefix_ls.adv_router = lsa->header->adv_router;
  abr_entry = ospf6_route_lookup (&prefix_ls, area->table_topology);

  prefix_ls.id         = ntohl (0);
  prefix_ls.adv_router = ier->router_id;

  request.destination = (struct prefix *) &prefix_ls;
  request.type = OSPF6_ROUTE_DEST_TYPE_ROUTER;
  request.capability = 0;
  request.prefix_options = 0;
  request.path.area_id = area->area_id;
  request.path.type = OSPF6_PATH_TYPE_INTER;
  request.path.cost = abr_entry->cost +
                      ntohl (ier->metric) & ntohl (0x000fffff);
  request.path.origin.type       = lsa->header->type;
  request.path.origin.id         = lsa->header->id;
  request.path.origin.adv_router = lsa->header->adv_router;
  request.nexthop.ifindex = abr_entry->ifindex;
  request.nexthop.address = abr_entry->address;

  ospf6_route_add (&request, ospf6->topology_table);
#endif /*0*/
  return 0;
}

int
ospf6_abr_inter_router_remove (void *data)
{
  return 0;
}

void
ospf6_abr_inter_router_update (struct ospf6_route *topology,
                               struct ospf6_area *area)
{
  char lsa_buffer[MAXLSASIZE];
  struct ospf6_inter_area_router_lsa *ier;
  struct ospf6_interface *o6i;
  int size;

  if (topology->path->type != OSPF6_PATH_TYPE_INTRA)
    return;

  if (topology->path->cost >= LS_INFINITY)
    return;

  /* if the area associated with this set of paths is the area
     itself, do not generate a summary-LSA for the route */
  if (topology->path->area_id == area->area_id)
    return;

  /* next hop's associated area is the area itself: logical
     equivalent of Distance Vector Split horizon */
  o6i = ospf6_interface_lookup_by_index (route->path->nexthop->ifindex);
  if (o6i->area->area_id == area->area_id)
    continue;

  /* if the area configured as stub */
  /* xxx */

  memset (lsa_buffer, 0, sizeof (lsa_buffer));
  ier = (struct ospf6_inter_area_router_lsa *) lsa_buffer;

  ier->options[0] = topology->capability[0];
  ier->options[1] = topology->capability[1];
  ier->options[2] = topology->capability[2];
  ier->metric     = htonl (topology->path->cost) & htonl (0x000fffff);
  ier->router_id  = ((struct prefix_ls *) &topology->prefix)->adv_router;

  size = sizeof (struct ospf6_inter_area_router_lsa);
  ospf6_lsa_originate (htons (OSPF6_LSA_TYPE_INTER_ROUTER),
                       htonl (topology->route_id), o6a->ospf6->router_id,
                       buffer, size, o6a);
}

int
ospf6_abr_inter_topology_add (void *data)
{
  struct ospf6_route *route = data;
  struct ospf6_area *o6a;
  listnode node;

  assert (route);
  assert (route->dest_type == OSPF6_DEST_TYPE_ROUTER);
  assert (route->path->path_type == OSPF6_PATH_TYPE_INTRA ||
          route->path->path_type == OSPF6_PATH_TYPE_INTER);
  assert (OSPF6_OPT_ISSET (topo_entry->capability, OSPF6_OPT_E));

  for (node = listhead (ospf6->area_list); node; nextnode (node))
    {
      o6a = getdata (node);
      ospf6_abr_inter_router_update (route, o6a);
    }
  return 0;
}

int
ospf6_abr_inter_topology_remove (void *data)
{
  struct ospf6_route *route = data;
  struct ospf6_area *o6a;

  assert (route);
  assert (route->dest_type == OSPF6_DEST_TYPE_ROUTER);
  assert (route->path->path_type == OSPF6_PATH_TYPE_INTRA ||
          route->path->path_type == OSPF6_PATH_TYPE_INTER);
  assert (OSPF6_OPT_ISSET (topo_entry->capability, OSPF6_OPT_E));

  for (node = listhead (ospf6->area_list); node; nextnode (node))
    {
      o6a = getdata (node);

      lsa = ospf6_lsdb_lookup_lsdb (htons (OSPF6_LSA_TYPE_INTER_ROUTER),
                                    htonl (route->route_id),
                                    o6a->ospf6->router_id, o6a->lsdb);
      if (lsa)
        ospf6_lsa_premature_aging (lsa);
    }
  return 0;
}

void
ospf6_abr_inter_router_show (struct vty *vty, struct ospf6_lsa *lsa)
{
  struct ospf6_lsa_inter_are_router_lsa *inter_router;
  char buf[128], router[128];

  assert (lsa->header);
  inter_router = (struct ospf6_lsa_inter_area_router_lsa *) (lsa->header + 1);

  ospf6_prefix_options_str (inter_router->options, buf, sizeof (buf));
  inet_ntop (AF_INET, inter_router->router_id, router, sizeof (router));

  vty_out (vty, "     Prefix Options: %s%s, buf, VTY_NEWLINE);
  vty_out (vty, "     MBZ: %#03x Metric: %5lu (%#05x)%s",
           0xfff00000 & nthol (inter_router->metric),
           0x000fffff & nthol (inter_router->metric),
           0x000fffff & nthol (inter_router->metric),
           VTY_NEWLINE)
  vty_out (vty, "     Router ID: %s%s, router, VTY_NEWLINE);
}

void
ospf6_abr_inter_router_refresh (void *old)
{
#if 0
  struct ospf6_route *route;
  struct ospf6_lsa *lsa = old;

  topology = ospf6_route_lookup_id (lsa->id, ospf6->topology_table);
  ospf6_abr_inter_router_update (topology, lsa->scope);
#endif /*0*/
}

void
ospf6_abr_register_inter_router ()
{
  struct ospf6_lsa_slot slot, *sp;
  struct ospf6_hook hook;

  memset (&slot, 0, sizeof (slot));
  slot.type         = htons (OSPF6_LSA_TYPE_INTER_ROUTER);
  slot.name         = "Inter-Router-LSA";
  slot.func_show    = ospf6_abr_inter_router_show;
  slot.func_refresh = ospf6_abr_inter_router_refresh;
  ospf6_lsa_slot_register (&slot);

  memset (&hook, 0, sizeof (hook));
  hook.name        = "CalculateInterRouter";
  hook.hook_add    = ospf6_abr_inter_router_add;
  hook.hook_change = ospf6_abr_inter_router_add;
  hook.hook_remove = ospf6_abr_inter_router_remove;
  sp = ospf6_lsa_slot_get (htons (OSPF6_LSA_TYPE_INTER_ROUTER));
  ospf6_hook_register (&hook, &sp->database_hook);

  memset (&hook, 0, sizeof (hook));
  hook.name        = "OriginateInterRouter";
  hook.hook_add    = ospf6_abr_inter_topology_add;
  hook.hook_change = ospf6_abr_inter_topology_add;
  hook.hook_remove = ospf6_abr_inter_topology_remove;
  ospf6_hook_register (&hook, &inter_topology_hook);
}

void
ospf6_abr_inter_prefix_show (struct ospf6_lsa *lsa)
{
}

void
ospf6_abr_inter_prefix_refresh (struct ospf6_lsa *lsa)
{
}

static int
ospf6_abr_inter_prefix_to_route (struct ospf6_route *route,
                                 struct ospf6_path *path,
                                 struct ospf6_lsa *lsa)
{
  struct ospf6_inter_area_prefix_lsa *iep;
  struct ospf6_area *area;
  struct prefix_ls prefix_ls;
  struct ospf6_route *abr_entry;

  if (lsa->header->type != htons (OSPF6_LSA_TYPE_INTER_PREFIX))
    return -1;

  if (IS_LSA_MAXAGE (lsa))
    return -1;

  memset (&prefix_ls, 0, sizeof (struct prefix_ls));
  prefix_ls.family = AF_UNSPEC;
  prefix_ls.prefixlen = 64; /* xxx */
  prefix_ls.id.s_addr = htonl (0);
  prefix_ls.adv_router.s_addr = lsa->header->adv_router;

  area = (struct ospf6_area *) lsa->scope;
  abr_entry = ospf6_route_lookup (&prefix_ls, area->table_topology);

  if (abr_entry == NULL)
    return -1;

  iep = (struct ospf6_inter_area_prefix_lsa *) (lsa->header + 1);

  memset (route, 0, sizeof (struct ospf6_route));
  memset (path, 0, sizeof (struct ospf6_path));
  route->path = path;

  route->dest_type = OSPF6_DEST_TYPE_NETWORK;
  route->prefix.family = AF_INET6;
  route->prefix.prefixlen = iep->prefix.prefix_length;
  ospf6_prefix_in6_addr (iep->prefix, &route->prefix.u.prefix6);

  route->path->cost = abr_entry->path->cost +
                      ntohl (iep->metric) & ntohl (0x000fffff);
  route->path->path_type = OSPF6_PATH_TYPE_INTER;
  route->path->origin_type = lsa->header->type;
  route->path->origin_id = lsa->header->id;
  route->path->origin_adv_router = lsa->header->adv_router;
  route->path->nexthop = abr_entry->path->nexthop;

  return 0;
}

int
ospf6_abr_inter_prefix_add (void *data)
{
  struct ospf6_lsa *lsa = data;
  struct ospf6_route route;
  struct ospf6_path path;

  if (ospf6_abr_inter_prefix_to_route (&route, &path, lsa) < 0)
    return -1;

  ospf6_route_add (&route, ospf6->route_table);
  return 0;
}

int
ospf6_abr_inter_prefix_remove (void *data)
{
  struct ospf6_lsa *lsa = data;
  struct ospf6_route route;
  struct ospf6_path path;

  if (ospf6_abr_inter_prefix_to_route (&route, &path, lsa) < 0)
    return -1;

  ospf6_route_remove (&route, ospf6->route_table);
  return 0;
}

#if 0
/* Intra-Route -> Inter-Area-Prefix-LSA */
static int
ospf6_abr_inter_prefix_to_lsa (char *lsa_buffer,
                               struct ospf6_route *route)
{
  struct ospf6_inter_area_prefix_lsa *iep;
  struct ospf6_interface *o6i;
  int size;

  if (route->path->type != OSPF6_PATH_TYPE_INTRA)
    return;

  if (topology->path->cost >= LS_INFINITY)
    return;

  /* if the area associated with this set of paths is the area
     itself, do not generate a summary-LSA for the route */
  if (topology->path->area_id == area->area_id)
    return;

  /* next hop's associated area is the area itself: logical
     equivalent of Distance Vector Split horizon */
  o6i = ospf6_interface_lookup_by_index (route->path->nexthop->ifindex);
  if (o6i->area->area_id == area->area_id)
    continue;

  /* if the area configured as stub */
  /* xxx */

  memset (lsa_buffer, 0, sizeof (lsa_buffer));
  iep = (struct ospf6_inter_area_prefix_lsa) lsa_buffer;

  iep->metric     = htonl (topology->path->cost) & htonl (0x000fffff);
  iep->prefix.prefix_length = route->prefix.prefixlen;
  iep->prefix.prefix_options = route->prefix_options;
  memcpy (iep->prefix + 1, &route->prefix.u.prefix6,
          OSPF6_PREFIX_SPACE (route->prefix.prefixlen));
  ospf6_prefix_apply_mask (&iep->prefix);

}

int
ospf6_abr_intra_route_add (void *data)
{
  struct ospf6_route *route = data;
  char lsa_buffer[MAXLSASIZE];
  int size;

  ospf6_abr_inter_prefix_to_lsa (lsa_buffer, route);
  size = sizeof (struct ospf6_inter_area_prefix_lsa);
  size += OSPF6_PREFIX_SPACE (route->prefix.prefixlen);

  ospf6_lsa_originate (htons (OSPF6_LSA_TYPE_INTER_PREFIX),
                       htonl (route->route_id), area->ospf6->router_id,
                       lsa_buffer, size, area);
}

int
ospf6_abr_intra_route_remove (void *data)
{
}
#endif /*0*/

void
ospf6_abr_register_inter_prefix ()
{
  struct ospf6_lsa_slot slot, *sp;
  struct ospf6_hook hook;

  memset (&slot, 0, sizeof (slot));
  slot.type         = htons (OSPF6_LSA_TYPE_INTER_PREFIX);
  slot.name         = "Inter-Prefix-LSA";
  slot.func_show    = ospf6_abr_inter_prefix_show;
  slot.func_refresh = ospf6_abr_inter_prefix_refresh;
  ospf6_lsa_slot_register (&slot);

  hook.name        = "CalculateInterPrefix";
  hook.hook_add    = ospf6_abr_inter_prefix_add;
  hook.hook_change = ospf6_abr_inter_prefix_add;
  hook.hook_remove = ospf6_abr_inter_prefix_remove;
  sp = ospf6_lsa_slot_get (htons (OSPF6_LSA_TYPE_INTER_PREFIX));
  ospf6_hook_register (&hook, &sp->database_hook);

  memset (&hook, 0, sizeof (hook));
  hook.name        = "OriginateInterPrefix";
#if 0
  hook.hook_add    = ospf6_abr_intra_route_add;
  hook.hook_change = ospf6_abr_intra_route_add;
  hook.hook_remove = ospf6_abr_intra_route_remove;
#endif /*0*/
  ospf6_hook_register (&hook, &route_hook);
}

void
ospf6_abr_init ()
{
  ospf6_abr_register_inter_router ();
  ospf6_abr_register_inter_prefix ();
}


